from OpenSSL import crypto


def get_keycert_from_p12(path, password):
    with open(path, "rb") as file:
        p12 = crypto.load_pkcs12(file.read(), password)
        # PEM formatted private key
        key = crypto.dump_privatekey(crypto.FILETYPE_PEM, p12.get_privatekey())
        # PEM formatted certificate
        cert = crypto.dump_certificate(
            crypto.FILETYPE_PEM, p12.get_certificate()
        )
        return key, cert



import xml.etree.ElementTree as ET
from lxml import etree
import os
import logging
from datetime import datetime
import pytbai
from string import Template
from signxml import DigestAlgorithm
from signxml.xades import (
    XAdESSigner,
    XAdESVerifier,
    XAdESVerifyResult,
    XAdESSignaturePolicy,
    XAdESDataObjectFormat,
)

logger = logging.getLogger("pytbai")


def build_xml(tbai, invoice):
    path = os.path.dirname(pytbai.__file__)
    structure_file = os.path.join(path, "templates/XML/tbai_structure.xml")
    with open(structure_file, "r") as file:
        template = file.read()
    temp = Template(template)

    expedition_date = datetime.strptime(
        invoice.expedition_date, "%Y-%m-%d"
    ).strftime("%d-%m-%Y")
    transaction_date = datetime.strptime(
        invoice.transaction_date, "%Y-%m-%d"
    ).strftime("%d-%m-%Y")

    xml = temp.substitute(
        version=tbai.version,
        entity_id=tbai.subject.entity_id,
        name=tbai.subject.name,
        multi_recipient=tbai.subject.multi_recipient,
        external_invoice=tbai.subject.external_invoice,
        serial_code=invoice.serial_code,
        num=invoice.num,
        description=invoice.description,
        simplified=invoice.simplified,
        substitution=invoice.substitution,
        vat_regime=invoice.vat_regime,
        expedition_date=expedition_date,
        expedition_time=invoice.expedition_time,
        transaction_date=transaction_date,
        license=tbai.software.license,
        dev_entity=tbai.software.dev_entity,
        soft_name=tbai.software.soft_name,
        soft_version=tbai.software.soft_version,
    )
    root = ET.fromstring(xml)
    lines = invoice.get_lines()
    lines_root = root.find(".//DetallesFactura")
    for line in lines:
        line_xml = ET.SubElement(lines_root, "IDDetalleFactura")
        desc_xml = ET.SubElement(line_xml, "DescripcionDetalle")
        desc_xml.text = line.description
        quantity_xml = ET.SubElement(line_xml, "Cantidad")
        quantity_xml.text = str(line.quantity)
        unit_amount_xml = ET.SubElement(line_xml, "ImporteUnitario")
        unit_amount_xml.text = str(line.unit_amount)
        if line.discount:
            line_base = line.get_line_base()
            discount_xml = ET.SubElement(line_xml, "Descuento")
            discount_xml.text = str(line.get_discount_qty(line_base))
        total_xml = ET.SubElement(line_xml, "ImporteTotal")
        total_xml.text = str(line.total)
    total_root = root.find(".//ImporteTotalFactura")
    total_root.text = str(invoice.get_total_amount())
    vat_type = root.find(".//NoExenta")
    breakdown = invoice.get_vat_breakdown()
    for vtype in breakdown:
        vtype_line = ET.SubElement(vat_type, "DetalleNoExenta")
        vbreakdown = ET.SubElement(vtype_line, "TipoNoExenta")
        vbreakdown.text = vtype["type"]
        vtbreak = ET.SubElement(vtype_line, "DesgloseIVA")
        for rate in vtype["rates"].items():
            rate_detail = ET.SubElement(vtbreak, "DetalleIVA")
            vat_base = ET.SubElement(rate_detail, "BaseImponible")
            vat_base.text = str(rate[1]["base"])
            vat_rate = ET.SubElement(rate_detail, "TipoImpositivo")
            vat_rate.text = str(float(rate[0]))
            vat_fee = ET.SubElement(rate_detail, "CuotaImpuesto")
            vat_fee.text = str(rate[1]["fee"])

    return root


def validate_xml(xml):
    path = os.path.dirname(pytbai.__file__)
    xsd_file = os.path.join(path, "templates/XSD/ticketBaiV1-2-1.xsd")
    with open(xsd_file, "r") as file:
        xmlschema_doc = etree.parse(file)
        xmlschema = etree.XMLSchema(xmlschema_doc)
        try:
            xmlschema.assert_(xml)
        except AssertionError as msg:
            logger.error(msg)
            return False
        return True


def sign_xml(xml, key, cert):
    signature_policy = XAdESSignaturePolicy(
        Identifier="https://www.gipuzkoa.eus/ticketbai/sinadura",
        Description="",
        DigestMethod=DigestAlgorithm.SHA256,
        DigestValue="vSe1CH7eAFVkGN0X2Y7Nl9XGUoBnziDA5BGUSsyt8mg=",
    )
    data_object_format = XAdESDataObjectFormat(
        Description="",
        MimeType="text/xml",
    )
    signer = XAdESSigner(
        signature_policy=signature_policy,
        claimed_roles=["signer"],
        data_object_format=data_object_format,
        c14n_algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315",
    )
    signed_doc = signer.sign(xml, key=key, cert=cert)
    return signed_doc



TBAI_JSON = {
    "version": "1.2",
    "subject": {
        "entity_id": "99999974E",
        "name": "REPRESENTANTESPJ FICTICIO",
        "authority_api": "https://tbai-z.prep.gipuzkoa.eus/sarrerak/alta",
        "qr_api": "https://tbai.egoitza.gipuzkoa.eus/qr",
        "multi_recipient": "N",
        "external_invoice": "N",
    },
    "software": {
        "license": "TBAIGIPRE00000000501",
        "dev_entity": "P2000000F",
        "soft_name": "FAKTURABAI",
        "soft_version": "1.0",
    },
}

TBAI_INVOICE_JSON = {
    "version": "1.2",
    "subject": {
        "entity_id": "99999974E",
        "name": "REPRESENTANTESPJ FICTICIO",
        "authority_api": "https://tbai-z.prep.gipuzkoa.eus/sarrerak/alta",
        "qr_api": "https://tbai.egoitza.gipuzkoa.eus/qr",
        "multi_recipient": "N",
        "external_invoice": "N",
    },
    "invoice": {
        "serial_code": "TB-2021-S",
        "num": 1,
        "description": "Primera factura",
        "expedition_date": "2023-07-05",
        "expedition_time": "12:58:17",
        "transaction_date": "2023-07-05",
        "simplified": "N",
        "substitution": "N",
        "vat_regime": "01",
        "lines": [],
    },
    "software": {
        "license": "TBAIGIPRE00000000501",
        "dev_entity": "P2000000F",
        "soft_name": "FAKTURABAI",
        "soft_version": "1.0",
    },
}

TBAI_INVOICE_LINES_JSON = {
    "version": "1.2",
    "subject": {
        "entity_id": "99999974E",
        "name": "REPRESENTANTESPJ FICTICIO",
        "authority_api": "https://tbai-z.prep.gipuzkoa.eus/sarrerak/alta",
        "qr_api": "https://tbai.egoitza.gipuzkoa.eus/qr",
        "multi_recipient": "N",
        "external_invoice": "N",
    },
    "invoice": {
        "serial_code": "TB-2021-S",
        "num": 1,
        "description": "Primera factura",
        "expedition_date": "2023-07-05",
        "expedition_time": "12:58:17",
        "transaction_date": "2023-07-05",
        "simplified": "N",
        "substitution": "N",
        "vat_regime": "01",
        "lines": [
            {
                "description": "Primer producto",
                "quantity": 1,
                "unit_amount": 200,
                "discount": 20,
                "vat_rate": 21,
                "vat_fee": 33.6,
                "vat_type": "S1",
                "vat_base": 160.0,
                "total": 193.6,
            },
            {
                "description": "Segundo producto",
                "quantity": 2,
                "unit_amount": 350,
                "discount": 0,
                "vat_rate": 21,
                "vat_fee": 147.0,
                "vat_type": "S1",
                "vat_base": 700,
                "total": 847.0,
            },
        ],
    },
    "software": {
        "license": "TBAIGIPRE00000000501",
        "dev_entity": "P2000000F",
        "soft_name": "FAKTURABAI",
        "soft_version": "1.0",
    },
}



from datetime import datetime
import tempfile
import json
import requests
import logging
from decimal import Decimal
from lxml import etree
from json import JSONEncoder
from decimal import Decimal
from pytbai.definitions import (
    TICKETBAI_ACTUAL_VERSION,
    DOCUMENTATION_URL,
    AUTHORITY_APIS,
    GIPUZKOA,
    DEFAULT_VAT_RATE,
    DEFAULT_VAT,
    N,
    L3,
    L4,
    L5,
    L6,
    L9,
    S1,
    L11,
)
from pytbai.utils.xml import build_xml, sign_xml, validate_xml
from pytbai.utils.crypto import get_keycert_from_p12
from pytbai.utils.pdf import build_pdf

logger = logging.getLogger("pytbai")


class Subject:
    def __init__(
        self,
        entity_id,
        name,
        territory=GIPUZKOA,
        multi_recipient=N,
        external_invoice=N,
    ):
        self.entity_id = entity_id
        self.name = name
        if not territory:
            self.authority_api = GIPUZKOA["invoice"]
            self.qr_api = GIPUZKOA["qr"]
        elif territory in AUTHORITY_APIS:
            self.authority_api = AUTHORITY_APIS[territory]["invoice"]
            self.qr_api = AUTHORITY_APIS[territory]["qr"]
        else:
            raise ValueError(
                "Not a valid territory. Options are: Araba, Bizkaia, Gipuzkoa."
            )
        if multi_recipient in L3:
            self.multi_recipient = multi_recipient
        else:
            raise ValueError(
                "Value not found in L3 options, see documentation: %s"
                % DOCUMENTATION_URL
            )
        if external_invoice in L4:
            self.external_invoice = external_invoice
        else:
            raise ValueError(
                "Value not found in L4 options, see documentation: %s"
                % DOCUMENTATION_URL
            )

    def get_dict(self):
        return self.__dict__


class InvoiceLine:
    def __init__(
        self,
        description,
        quantity=0,
        unit_amount=0,
        discount=0,
        vat_rate=DEFAULT_VAT_RATE,
        vat_type=None,
    ):
        self.description = description
        self.quantity = quantity
        self.unit_amount = unit_amount
        self.discount = discount
        self.vat_rate = vat_rate
        self.vat_fee = None
        if not vat_type:
            self.vat_type = S1
        elif vat_type in L11:
            self.vat_type = vat_type
        else:
            raise ValueError(
                "Value not found in L11 options, see documentation: %s"
                % DOCUMENTATION_URL
            )

        self.set_base()
        self.set_vat_fee()
        self.set_total()

    def get_line_base(self):
        line_base = self.quantity * self.unit_amount
        return line_base.quantize(Decimal("0.00"))

    def get_discount_qty(self, line_base):
        discount_qty = line_base * (self.discount / 100)
        return discount_qty.quantize(Decimal("0.00"))

    def set_base(self):
        line_base = self.get_line_base()
        if self.discount:
            self.vat_base = line_base - self.get_discount_qty(line_base)
        else:
            self.vat_base = line_base

    def set_vat_fee(self):
        vat_fee = self.vat_base * (self.vat_rate / 100)
        self.vat_fee = vat_fee.quantize(Decimal("0.00"))

    def set_total(self):
        self.total = self.vat_base + self.vat_fee

    def get_dict(self):
        return self.__dict__


class Invoice:
    def __init__(
        self,
        serial_code,
        num,
        description,
        simplified=None,
        substitution=None,
        vat_regime=None,
        expedition_date=None,
        expedition_time=None,
        transaction_date=None,
    ):

        self.serial_code = serial_code
        self.num = num
        self.description = description
        self.expedition_date = (
            expedition_date or datetime.now().date().isoformat()
        )
        self.expedition_time = (
            expedition_time or datetime.now().time().strftime("%H:%M:%S")
        )
        self.transaction_date = (
            transaction_date or datetime.now().date().isoformat()
        )

        if not simplified:
            self.simplified = N
        elif simplified in L5:
            self.simplified = simplified
        else:
            raise ValueError(
                "Value not found in L5 options, see documentation: %s"
                % DOCUMENTATION_URL
            )

        if not substitution:
            self.substitution = N
        elif substitution in L6:
            self.substitution = substitution
        else:
            raise ValueError(
                "Value not found in L6 options, see documentation: %s"
                % DOCUMENTATION_URL
            )

        if not vat_regime:
            self.vat_regime = DEFAULT_VAT
        elif vat_regime in L9:
            self.vat_regime = vat_regime
        else:
            raise ValueError(
                "Value not found in L9 options, see documentation: %s"
                % DOCUMENTATION_URL
            )

        self.lines = []

    def get_lines(self):
        return self.lines

    def get_total_amount(self):
        lines = self.get_lines()
        total = sum([line.total for line in lines])
        return total.quantize(Decimal("0.00"))

    def get_vat_breakdown(self):
        breakdown = []
        for vat_type in L11:
            lines = [
                line for line in self.get_lines() if line.vat_type == vat_type
            ]
            if lines:
                line_types = {"type": vat_type, "rates": {}}
                for line in lines:
                    if line.vat_rate in line_types["rates"]:
                        line_types["rates"][line.vat_rate] = {
                            "base": line_types["rates"][line.vat_rate]["base"]
                            + line.vat_base,
                            "fee": line_types["rates"][line.vat_rate]["fee"]
                            + line.vat_fee,
                        }
                    else:
                        line_types["rates"].update(
                            {
                                line.vat_rate: {
                                    "base": line.vat_base,
                                    "fee": line.vat_fee,
                                }
                            }
                        )
                breakdown.append(line_types)
        return breakdown

    def create_line(
        self,
        description,
        quantity=0,
        unit_import=0,
        discount=0,
        vat_rate=DEFAULT_VAT_RATE,
        vat_type=S1,
    ):
        line = InvoiceLine(
            description,
            quantity,
            unit_import,
            discount,
            vat_rate,
            vat_type,
        )
        self.lines.append(line)

    def delete_lines(self, lines):
        curr_lines = self.lines
        for line in lines:
            curr_lines.remove(line)
        self.lines = curr_lines

    def get_dict(self):
        invoice_json = self.__dict__
        lines_json = []
        for line in invoice_json["lines"]:
            lines_json.append(line.get_dict())
        invoice_json["lines"] = lines_json
        return invoice_json


class Software:
    def __init__(
        self,
        license,
        dev_entity,
        soft_name,
        soft_version,
    ):
        self.license = license
        self.dev_entity = dev_entity
        self.soft_name = soft_name
        self.soft_version = soft_version

    def get_dict(self):
        return self.__dict__


class TBaiEncoder(JSONEncoder):
    def default(self, o):
        if isinstance(o, Decimal):
            return float(o)
        return super().default(o)


class TBai:
    def __init__(self, config, version=TICKETBAI_ACTUAL_VERSION):
        self.version = version
        self.subject = Subject(**config["subject"])
        self.software = Software(**config["software"])

    def create_invoice(
        self,
        serial_code,
        num,
        description,
        simplified=None,
        expedition_date=None,
        expedition_time=None,
        transaction_date=None,
    ):
        invoice = Invoice(
            serial_code,
            num,
            description,
            simplified=simplified,
            expedition_date=expedition_date
            or datetime.now().date().isoformat(),
            expedition_time=expedition_time
            or datetime.now().time().strftime("%H:%M:%S"),
            transaction_date=transaction_date
            or datetime.now().date().isoformat(),
        )
        return invoice

    def sign_and_send(self, invoice, p12_path, password):
        xml = build_xml(self, invoice)
        key, cert = get_keycert_from_p12(p12_path, password.encode("utf-8"))
        signed_xml = sign_xml(xml, key, cert)
        if not validate_xml(signed_xml):
            return (None, None)

        key_file = tempfile.NamedTemporaryFile()
        key_file.write(key)
        key_file.flush()
        cert_file = tempfile.NamedTemporaryFile()
        cert_file.write(cert)
        cert_file.flush()

        headers = {"Content-Type": "application/xml"}

        # TODO: Error handling for timeout
        response = requests.post(
            url=self.subject.authority_api,
            headers=headers,
            data=etree.tostring(signed_xml),
            cert=(cert_file.name, key_file.name),
        )

        key_file.close()
        cert_file.close()

        result_json = {
            "TBAI_ID": None,
            "CSV": None,
            "SignedXML": None,
            "ResponseXML": None,
        }
        if response.ok:
            response_xml = etree.fromstring(response.content)
            state = response_xml.find(".//Estado").text
            if state == "01":
                logger.error("XML not accepted")
                result_json["ResponseXML"] = response_xml
                return result_json
            result_json["TBAI_ID"] = response_xml.find(
                ".//IdentificadorTBAI"
            ).text
            result_json["CSV"] = response_xml.find(".//CSV").text
            result_json["SignedXML"] = etree.tostring(signed_xml).decode(
                "utf-8"
            )
            return result_json
        logger.error("API connection error")
        return result_json

    def get_json(self, invoice=None):
        tbai_json = {"version": self.version}
        if self.subject:
            tbai_json["subject"] = self.subject.get_dict()
        if invoice:
            tbai_json["invoice"] = invoice.get_dict()
        if self.software:
            tbai_json["software"] = self.software.get_dict()
        return json.dumps(tbai_json, cls=TBaiEncoder)



import os
import pytbai
import qrcode
import base64
from io import BytesIO
from string import Template
from requests.models import PreparedRequest
from weasyprint import HTML, CSS

TEMPLATES_PATH = os.path.join(os.path.dirname(pytbai.__file__), "templates")


def create_qr_base64(invoice, tbai_id, subject):
    request = PreparedRequest()
    params = {
        "id": tbai_id,
        "s": invoice.serial_code,
        "nf": invoice.num,
        "i": invoice.get_total_amount(),
        "cr": tbai_id.split("-")[-1],
    }
    request.prepare_url(subject.qr_api, params)

    qr_code = qrcode.QRCode(box_size=3)
    qr_code.add_data("%s" % request.url)
    img = qr_code.make_image(fill_color="black", back_color="white")
    buffered = BytesIO()
    img.save(buffered, format="JPEG")
    img.close()
    return base64.b64encode(buffered.getvalue())


def get_css_string():
    css_file = os.path.join(TEMPLATES_PATH, "PDF/ticketbai.css")
    with open(css_file, "r") as file:
        css_string = file.read()
        return css_string


def get_html_string(invoice, tbai_id, subject):
    html_file = os.path.join(TEMPLATES_PATH, "PDF/ticketbai.html")
    qr_base64 = create_qr_base64(invoice, tbai_id, subject)
    subject_info = "%s</br>IFZ: %s</br>" % (subject.name, subject.entity_id)
    date = invoice.expedition_date.strftime("%Y-%m-%d")
    hour = invoice.expedition_time.strftime("%H:%M:%S")
    serial_code = invoice.serial_code
    num = invoice.num
    invoice_total = str(invoice.get_total_amount()).replace(".", ",")

    with open(html_file, "r") as file:
        template = file.read()
    temp = Template(template)
    return temp.substitute(
        subject_info=subject_info,
        date=date,
        hour=hour,
        serial_code=serial_code,
        num=num,
        invoice_total=invoice_total,
        tbai_id=tbai_id,
        qrcode=qr_base64.decode("utf-8"),
    )


def build_pdf(invoice, tbai_id, subject):
    css = CSS(string=get_css_string())
    html = HTML(string=get_html_string(invoice, tbai_id, subject))
    return html.write_pdf(stylesheets=[css])



# -*- coding: utf-8 -*-

import pytbai
from pytbai import TBai
from pytbai.utils.xml import build_xml, validate_xml, sign_xml
import os
import json
import unittest
from datetime import datetime
from decimal import Decimal
from pytbai.utils.crypto import get_keycert_from_p12
from tests.data.tbai_json import (
    TBAI_JSON,
    TBAI_INVOICE_JSON,
    TBAI_INVOICE_LINES_JSON,
)

CONFIG = {
    "subject": {"entity_id": "99999974E", "name": "REPRESENTANTESPJ FICTICIO"},
    "software": {
        "license": "TBAIGIPRE00000000501",
        "dev_entity": "P2000000F",
        "soft_name": "FAKTURABAI",
        "soft_version": "1.0",
    },
}


class BasicTestSuite(unittest.TestCase):
    """Basic test cases."""

    def test_tbai_subject_constructor(self):
        tbai = TBai(CONFIG)
        self.assertIsNotNone(tbai.subject)
        self.assertEqual(tbai.subject.entity_id, "99999974E")
        self.assertEqual(tbai.subject.name, "REPRESENTANTESPJ FICTICIO")

    def test_tbai_software_constructor(self):
        tbai = TBai(CONFIG)
        self.assertIsNotNone(tbai.software)
        self.assertEqual(tbai.software.license, "TBAIGIPRE00000000501")
        self.assertEqual(tbai.software.dev_entity, "P2000000F")
        self.assertEqual(tbai.software.soft_name, "FAKTURABAI")
        self.assertEqual(tbai.software.soft_version, "1.0")

    def test_create_invoice(self):
        tbai = TBai(CONFIG)
        invoice = tbai.create_invoice("TB-2021-S", 1, "Primera factura")
        self.assertIsNotNone(invoice)
        self.assertEqual(invoice.serial_code, "TB-2021-S")
        self.assertEqual(invoice.num, 1)
        self.assertEqual(invoice.description, "Primera factura")
        self.assertEqual(invoice.simplified, "N")
        self.assertEqual(invoice.substitution, "N")
        self.assertEqual(invoice.vat_regime, "01")

    def test_create_invoice_line(self):
        tbai = TBai(CONFIG)
        invoice = tbai.create_invoice("TB-2021-S", 1, "Primera factura")
        invoice.create_line(
            "Primer producto", Decimal("1"), Decimal("200"), Decimal("20")
        )
        invoice.create_line("Segundo producto", Decimal("2"), Decimal("350"))
        self.assertTrue(invoice.get_lines())
        lines = invoice.get_lines()
        self.assertEqual(lines[0].vat_base, Decimal("160"))
        self.assertEqual(lines[0].total, Decimal("193.6"))
        self.assertEqual(lines[1].vat_base, Decimal("700"))
        self.assertEqual(lines[1].total, Decimal("847"))

    def test_build_xml(self):
        tbai = TBai(CONFIG)
        invoice = tbai.create_invoice("TB-2021-S", 1, "Primera factura")
        invoice.create_line(
            "Primer producto", Decimal("1"), Decimal("200"), Decimal("20")
        )
        invoice.create_line("Segundo producto", Decimal("2"), Decimal("350"))
        xml = build_xml(tbai, invoice)
        self.assertIsNotNone(xml)

    def test_sign_validate_xml(self):
        path = os.path.dirname(pytbai.__file__)
        cert_path = os.path.join(path, "../tests/certs/cert_for_tests.p12")
        tbai = TBai(CONFIG)
        invoice = tbai.create_invoice("TB-2021-S", 1, "Primera factura")
        invoice.create_line(
            "Primer producto", Decimal("1"), Decimal("200"), Decimal("20")
        )
        invoice.create_line("Segundo producto", Decimal("2"), Decimal("350"))
        xml = build_xml(tbai, invoice)
        key, cert = get_keycert_from_p12(cert_path, b"testpassword")
        signed_xml = sign_xml(xml, key, cert)
        result = validate_xml(signed_xml)
        self.assertEqual(result, True)

    def test_get_json(self):
        tbai = TBai(CONFIG)
        date = datetime(2023, 7, 5, 12, 58, 17, 710211)
        invoice = tbai.create_invoice(
            "TB-2021-S",
            1,
            "Primera factura",
            expedition_date=date.date().isoformat(),
            expedition_time=date.time().strftime("%H:%M:%S"),
            transaction_date=date.date().isoformat(),
        )
        tbai_json = tbai.get_json()
        self.assertDictEqual(json.loads(tbai_json), TBAI_JSON)
        tbai_json = tbai.get_json(invoice)
        self.assertDictEqual(json.loads(tbai_json), TBAI_INVOICE_JSON)
        invoice.create_line(
            "Primer producto", Decimal("1"), Decimal("200"), Decimal("20")
        )
        invoice.create_line("Segundo producto", Decimal("2"), Decimal("350"))
        tbai_json = tbai.get_json(invoice)
        self.assertDictEqual(json.loads(tbai_json), TBAI_INVOICE_LINES_JSON)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open("README.md") as f:
    readme = f.read()

with open("LICENSE") as f:
    license = f.read()

setup(
    name="pytbai",
    version="1.2",
    description=(
        "pytbai allows to create, manage and send TicketBai invoices to the"
        " Basque tax authorities"
    ),
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Urtzi Odriozola",
    author_email="uodriozola@codesyntax.com",
    url="https://github.com/codesyntax/pytbai",
    license=license,
    packages=find_packages(exclude=("tests", "docs")),
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)



from decimal import Decimal

TICKETBAI_ACTUAL_VERSION = "1.2"
DOCUMENTATION_URL = "https://www.gipuzkoa.eus/eu/web/ogasuna/ticketbai/dokumentazioa-eta-araudia"

DEFAULT_VAT_RATE = Decimal("21")

GIPUZKOA = "Gipuzkoa"
AUTHORITY_APIS = {
    "Araba": "",
    "Bizkaia": "",
    GIPUZKOA: {
        "invoice": "https://tbai-z.prep.gipuzkoa.eus/sarrerak/alta",
        "qr": "https://tbai.egoitza.gipuzkoa.eus/qr",
    },
}

S = "S"  # Bai
N = "N"  # Ez

# Hainbat hartzaile
L3 = [
    S,
    N,
]

L4 = [
    "N",  # Jaulkitzaileak berak egindako faktura
    "T",  # Hirugarrenak egindako faktura
    "D",  # Eragiketaren hartzaileak egindako faktura
]

# Faktura sinplifikatua
L5 = [
    S,
    N,
]

# Faktura sinplifikatuaren ordez egindako faktura
L6 = [
    S,
    N,
]

DEFAULT_VAT = "01"
# BEZaren eta zerga garrantzia duten eragiketen erregimen bereziko gakoa
L9 = [
    DEFAULT_VAT,  # Erregimen orokorreko eragiketa eta hurrengo balioetan jaso gabe dagoen beste edozein kasu
    "02",  # Esportazioa
    "03",  # Erabilitako ondasunen, arte objektuen, zaharkinen eta bilduma objektuen araudi berezia aplikatzen zaien eragiketak
    "04",  # Inbertsio urrearen erregimen berezia
    "05",  # Bidaia agentzien erregimen berezia
    "06",  # BEZeko entitateen multzoaren erregimen berezia (maila aurreratua)
    "07",  # Kutxa irizpidearen erregimen berezia
    "08",  # Ekoizpen, zerbitzu eta inportazioaren gaineko zergari edo kanarietako zeharkako zerga orokorrari lotutako eragiketak
    "09",  # Besteren izenean eta kontura ari diren bidai agentziek egindako zerbitzuen fakturazioa (Fakturazio Erregelamenduko 3. xedapen gehigarria)
    "10",  # Hirugarrenen kontura kobratzea ordainsari profesionalak edo jabetza industrialetik eratorritako eskubideak, egilearenak edo bazkideen, elkartekideen edo elkargokideen kontura kobratzeko eginkizun horiek betetzen dituzten sozietate, elkarte, elkargo profesional edo bestelako entitateek egindakoak
    "11",  # Negozio lokala errentatzeko eragiketak, atxikipenari lotuak
    "12",  # Negozio lokala errentatzeko eragiketak, atxikipenari lotu gabeak
    "13",  # Negozio lokala errentatzeko eragiketak, atxikipenari lotuak eta lotu gabeak
    "14",  # Hartzailea administrazio publiko bat denean ordaintzeke dauden BEZdun fakturak, obra Ziurtagirietakoak
    "15",  # Segidako traktuko eragiketetan ordaintzeke dagoen BEZdun faktura
    "17",  # IX. tituluko XI. kapituluan aurreikusitako araubideetako batera (OSS edo IOSS) bildutako eragiketa
    "19",  # Nezakaritza, abeltzaintza eta arrantzako araubide berezian (NAAAB) dauden jardueren eragiketak
    "51",  # Baliokidetasun errekarguko eragiketak
    "52",  # Erregimen sinplifikatuko eragiketak
    "53",  # BEZaren ondorioetarako enpresari edo profesionaltzat jotzen ez diren pertsona edo entitateek egindako eragiketak
]


# Salbuetsi gabeko eragiketaren mota
S1 = "S1"
L11 = [
    S1,  # Subjektu pasiboaren inbertsiorik gabe
    "S2",  # Subjektu pasiboaren inbertsioarekin
]
